#define ZASM_HASH_TABLE_C 1

#include <zasm/hash_table.h>
#include <zasm/memory.h>
#include <zasm/string.h>
#include <string.h>

static unsigned int hash(char *name)
{
  unsigned int h = 5381;
  int i;

  for (i = 0; name[i]; i++)
    h = ((h<<5)-h)+(string_case(name[i]));
  h &= (ZASM_HASH_TABLE_SIZE-1);
  return h;
}

struct hash_table_t *hash_table_new()
{
  struct hash_table_t *val;
  
  ALLOCATE(struct hash_table_t, val, 1);
  hash_table_fill(val);
  return val;
}

void hash_table_fill(struct hash_table_t *table)
{
  int i;

  for (i = 0; i < ZASM_HASH_TABLE_SIZE; i++)
    table->table[i] = NULL;
}

struct hash_entry_t **hash_table_lookup(struct hash_table_t *table, char *name)
{
  int h;
  struct hash_entry_t **val;

  h = hash(name);
  for (val = &(table->table[h]); *val; val = &((*val)->next))
    if (0==string_compare_case((*val)->name, name))
      return val;
  return val;
}

void hash_table_delete(struct hash_table_t *table)
{}

void hash_table_free(struct hash_table_t *table)
{
  free(table);
}

void hash_entry_fill(struct hash_entry_t *entry, char *name)
{
  string_change_case(name);
  entry->name = name;
  entry->next = NULL;
}

void hash_entry_unlink(struct hash_entry_t **entry)
{
  *entry = (*entry)->next;
}

void hash_entry_delete(struct hash_entry_t *entry)
{}
