#define ZASM_LINK_GLOBAL_C 1

#include <zasm/link_expr.h>
#include <zasm/link_global.h>

ALLOC_DEFINE(link_global, ALLOC_DEF_COUNT);

struct link_global_t *link_global_new(struct link_level_t *level, struct link_entry_t *g_entry, struct link_entry_t *l_entry)
{
  struct link_global_t *val;

  val = link_global_alloc_get(&(level->global_alloc));
  link_global_fill(val, level, g_entry, l_entry);
  return val;
}

void link_global_fill(struct link_global_t *global, struct link_level_t *level,
                      struct link_entry_t *g_entry, struct link_entry_t *l_entry)
{
  global->global = g_entry;
  global->local = l_entry;
  global->expr = link_expr_new(level);
  global->expr->type = EXPR_VALUE;
  global->expr->value.entry = l_entry;

  global->next = level->globals;
  level->globals = global;
}

void link_global_copy(struct link_global_t *global, struct link_level_t *dest, struct link_level_t *global_level)
{
  struct link_global_t *new;
  struct link_entry_t *l_entry;
  struct link_expr_t *expr;

  if (!dest)
    return;

  for (; global; global = global->next)
    {
      link_expr_simplify(global->expr);
      if (global->expr->type == EXPR_VALUE)
	{
	  expr = link_expr_copy(global->expr, global_level);
	}
      else
	{
	  expr = link_expr_copy(global->expr, dest);
	  new = link_global_alloc_get(&(dest->global_alloc));
	  new->global = global->global;
	  new->expr = expr;
	  new->next = dest->globals;
	  dest->globals = new;
	}
      global->global->expr = expr;
      if (link_entry_defined(global->local))
	link_entry_define_global(global->global, global->local->page);
    }
}

void link_global_delete(struct link_global_t *global)
{}

void link_global_free(struct link_global_t *global)
{}
